  PAGE    60,132
  TITLE   fxSE.ASM -- PCX F/X
  SUBTTL  Copyright (c) Genus Microprogramming, Inc. 1988-89

; fxSE.ASM                                                                   ;
; Copyright (c) Genus Microprogramming, Inc. 1988-89  All Rights Reserved.   ;

;****************************************************************************;
;                                                                            ;
; This file contains procedures for setting the display/resolution to use    ;
; when displaying pcx images with the library routines.                      ;
;                                                                            ;
; Procedures: fxSetEffect                                                    ;
;             fxInitData                                                     ;
;                                                                            ;
; Microsoft ASM 5.x version.              Programmer: Chris Howard  3/15/89  ;
;                                                                            ;
;****************************************************************************;

; Include files
  INCLUDE ..\inc\pcxDefs.inc
  INCLUDE ..\inc\pcxMacs.inc
  INCLUDE ..\inc\pcxErrs.inc

  INCLUDE ..\inc\fxDefs.inc
  INCLUDE ..\inc\fxMacs.inc
  INCLUDE ..\inc\fxErrs.inc

  @SetModel

  @BegData

  IFDEF TURBOP

          EXTRN     fxVer                   : BYTE

          EXTRN     fx100th                 : WORD
          EXTRN     fxKeyChk                : WORD
          EXTRN     fxRand                  : DWORD

          EXTRN     fxVinit                 : BYTE

  ELSE

fxID      db        " PCX Effects "
fxVer     db        fxVERNUM
genusID   db        " Copyright (c) Genus Microprogramming, Inc. 1988-89"
          db        " All Rights Reserved."
progID    db        " Christopher A. Howard ",ASCIIZ

          PUBLIC    fxVinit,fx100th,fxKeyChk,fxRand

fxVinit   db        pcxFALSE
fx100th   dw        500                     ;Current 1/100 second loop delay
fxKeyChk  dw        pcxFALSE                ;Key Check Abort flag
fxRand    dd        ?                       ;Random number seed

  ENDIF

fxEffect  dw        fxWIPE                  ;Current effect type
fxGrain   dw        8                       ;Current granularity
fxDelay   dw        0                       ;Current delay

fxEffects fxstruc   <fxWIPE   ,"Wipe                ">
fxlen     EQU       $-fxEffects
          fxstruc   <fxSPLIT  ,"Split               ">
          fxstruc   <fxCRUSH  ,"Crush               ">
          fxstruc   <fxSLIDE  ,"Slide               ">
          fxstruc   <fxSAND   ,"Sand                ">
          fxstruc   <fxDRIP   ,"Drip                ">
          fxstruc   <fxEXPLODE,"Explode             ">
          fxstruc   <fxDIAG   ,"Diagonal            ">
          fxstruc   <fxSPIRAL ,"Spiral              ">
          fxstruc   <fxRANDOM ,"Random              ">

  @EndData

  @BegCode

          EXTRN     fxSetRand               : FAR

          PUBLIC    fxSetEffect,fxGetEffect
          PUBLIC    fxSetGrain, fxGetGrain
          PUBLIC    fxSetDelay, fxGetDelay
          PUBLIC    fxGetVersion
          PUBLIC    fxGetEffectStruc
          PUBLIC    fxInitData

;**********

;
; This procedure records the special effect type, and other effect parameters
; used when manipulating images.
;
; Calling: retcode = fxSetEffect(int fxtype)
;
;

;Define variable locations on the stack  (pascal model)
seeffect  equ       <[bp+6]>
separm    equ       2

;Define local variables
seret     equ       <[bp-2]>                ;return code
selocal   equ       2                       ;Total local space needed

fxSetEffect         PROC FAR

          @Entry    selocal                 ;Set up frame and save regs

          mov       ax,seeffect             ;Get the requested effect type
          cmp       ax,mineffect            ;Check lower limit
          jb        fxSE_badeffect

          cmp       ax,maxeffect            ;Check upper limit
          jbe       fxSE_seteffect

fxSE_badeffect:

          @SetRet   seret,fxERR_BADEFFECT   ;Bad effect type
          jmp       SHORT fxSE_exit

fxSE_seteffect:

          mov       fxEffect,ax             ;Store the effect type

          cmp       ax,fxRANDOM             ;Is this a random effect?
          jne       fxSE_ok                 ;If not, exit

          @CLOCK    READCLOCK               ;Read the current clock count

          push      dx                      ;Set a random number seed
          call      fxSetRand

fxSE_ok:

          @SetRet   seret,fxSUCCESS         ; and set successful code

fxSE_exit:

          @Exit     seret,separm            ;Return

fxSetEffect         ENDP

;**********

;
; This procedure gets the effect type previously set by the fxSetEffect
; call.
;
; Calling: effect = fxGetEffect()
;
;

;Define variable locations on the stack, depending on model
geparm    equ       0

;Define local variables
geret     equ       <[bp-2]>                ;return code
gelocal   equ       2                       ;Total local space needed

fxGetEffect         PROC FAR

          @Entry    gelocal                 ;Set up frame and save regs

          mov       ax,fxEffect             ;Get the current effect
          @SetRet   geret,ax                ; and set the return code

          cmp       ax,mineffect            ;Is this a valid number?
          jl        fxGE_error              ;If not, set error code
          cmp       ax,maxeffect            ;Check upper limit
          jle       fxGE_exit               ; and if less than or equal, ok

fxGE_error:

          @SetRet   geret,fxERR_BADEFFECT   ;Bad Effect . . .

fxGE_exit:

          @Exit     geret,geparm            ;Return

fxGetEffect         ENDP

;**********

;
; This procedure records the granularity to be used for subsequent effects.  
;
; Calling: retcode = fxSetGrain(int grain)
;
;

;Define variable locations on the stack  (pascal model)
sggrain   equ       <[bp+6]>
sgparm    equ       2

;Define local variables
sgret     equ       <[bp-2]>                ;return code
sglocal   equ       2                       ;Total local space needed

fxSetGrain          PROC FAR

          @Entry    sglocal                 ;Set up frame and save regs

          mov       ax,sggrain              ;Get the requested grain      
          cmp       ax,MINGRAIN             ;Check lower limit
          jb        fxSG_badgrain  

          cmp       ax,MAXGRAIN             ;Check upper limit
          jbe       fxSG_setgrain 

fxSG_badgrain:

          @SetRet   sgret,fxERR_BADGRAIN    ;Bad grain type
          jmp       SHORT fxSG_exit

fxSG_setgrain:

          mov       fxGrain,ax              ;Store the effect type
          @SetRet   sgret,fxSUCCESS         ; and set successful code

fxSG_exit:

          @Exit     sgret,sgparm            ;Return

fxSetGrain          ENDP

;**********

;
; This procedure gets the grain previously set by the fxSetGrain 
; call.
;
; Calling: grain = fxGetGrain()
;
;

;Define variable locations on the stack, depending on model
ggparm    equ       0

;Define local variables
ggret     equ       <[bp-2]>                ;return code
gglocal   equ       2                       ;Total local space needed

fxGetGrain          PROC FAR

          @Entry    gglocal                 ;Set up frame and save regs

          mov       ax,fxGrain              ;Get the current grain 
          @SetRet   ggret,ax                ; and set the return code

fxGG_exit:

          @Exit     ggret,ggparm            ;Return

fxGetGrain          ENDP

;**********

;
; This procedure records the delay to be used for subsequent effects.  
;
; Calling: retcode = fxSetDelay(int delay)
;
;

;Define variable locations on the stack  (pascal model)
sdgrain   equ       <[bp+6]>
sdparm    equ       2

;Define local variables
sdret     equ       <[bp-2]>                ;return code
sdlocal   equ       2                       ;Total local space needed

fxSetDelay          PROC FAR

          @Entry    sdlocal                 ;Set up frame and save regs

          mov       ax,sdgrain              ;Get the requested grain      
          cmp       ax,MINDELAY             ;Check lower limit
          jb        fxSD_baddelay  

          cmp       ax,MAXDELAY             ;Check upper limit
          jbe       fxSD_setdelay 

fxSD_baddelay:

          @SetRet   sdret,fxERR_BADDELAY    ;Bad grain type
          jmp       SHORT fxSD_exit

fxSD_setdelay:

          mov       fxDelay,ax              ;Store the effect type
          @SetRet   sdret,fxSUCCESS         ; and set successful code

fxSD_exit:

          @Exit     sdret,sdparm            ;Return

fxSetDelay          ENDP

;**********

;
; This procedure gets the delay previously set by the fxSetDelay 
; call.
;
; Calling: grain = fxGetDelay()
;
;

;Define variable locations on the stack, depending on model
gdparm    equ       0

;Define local variables
gdret     equ       <[bp-2]>                ;return code
gdlocal   equ       2                       ;Total local space needed

fxGetDelay          PROC FAR

          @Entry    gdlocal                 ;Set up frame and save regs

          mov       ax,fxDelay              ;Get the current delay 
          @SetRet   gdret,ax                ; and set the return code

fxGD_exit:

          @Exit     gdret,gdparm            ;Return

fxGetDelay          ENDP

;**********

;
; This procedure gets the current PCX Effects version.
;
; Calling: retcode = fxGetVersion(char *fxver)
;
;

;Define variable locations on the stack, depending on model
gvverseg  equ       <[bp+ 8]>
gvverofs  equ       <[bp+ 6]>
gvparm    equ       4

;Define local variables
gvret     equ       <[bp-2]>                ;return code
gvlocal   equ       2                       ;Total local space needed

fxGetVersion        PROC FAR

          @Entry    gvlocal                 ;Set up frame and save regs

          @@LoadSeg es,gvverseg             ;Point to the version buffer
          mov       di,gvverofs

          @@Data    ds                      ;Point to the version string
          mov       si,OFFSET fxVER

          mov       cx,4                    ;Length of version is always 4

          cld
          rep       movsb

          mov       BYTE PTR es:[di],0      ;End it

          @SetRet   gvret,fxSUCCESS         ; and set return code

fxGV_exit:

          @Exit     gvret,gvparm            ;Return

fxGetVersion        ENDP

;**********

;
; This procedure returns the effect structure for the requested effect type.
;
; Calling: strucptr = fxGetEffectStruc(disp)
;
; Returns: DX    Segment
;          AX    Offset
;
; NOTES: - Return in DX and AX, so that current pointers are not destroyed
;

;Define variable locations on the stack, depending on model
gseffect  equ       <[bp+ 6]>
gsparm    equ       2

;Define local variables
gsret     equ       <[bp-2]>                ;return code
gslocal   equ       2                       ;Total local space needed

fxGetEffectStruc    PROC FAR

          @Entry    gslocal                 ;Set up frame and save regs

          mov       ax,gseffect             ;Get the display type requested
          cmp       ax,mineffect            ;Check lower limit
          jb        fxGS_badeffect

          cmp       ax,maxeffect            ;Check upper limit
          jbe       fxGS_getstruc

fxGS_badeffect:

          @SetRet   gsret,fxERR_BADEFFECT   ;Bad effect type
          jmp       SHORT fxGS_exit

fxGS_getstruc:

          mov       di,OFFSET fxEffects     ;Get offset of table
          mov       ax,fxlen                ; and length of an entry

          mov       cx,gseffect             ;Multiply by effect type
          mul       cx

          add       di,ax                   ;Index into table
          @@Data    es                      ; (ES:DI)

          mov       ax,di                   ;Store for return values
          mov       dx,es                   ; (DX:AX)

;
; Do not verify the entry, since this is used when initializing the
; structures (and the entry is undefined ...)
;
;         mov       bl,es:[di].fxtype       ;Check the first entry
;         cmp       bl,gseffect             ;Is it the requested effect?
;         je        fxGS_success            ;If so, we're ok
;
;         @SetRet   gsret,fxERR_GENERAL     ;General Error
;         jmp       SHORT fxGS_exit

fxGS_success:

          @SetRet   gsret,ax                ;Successful, so don't destroy ptr

fxGS_exit:

          @Exit     gsret,gsparm            ;Return

fxGetEffectStruc    ENDP

;**********

;
; This procedure is used to initialize all of the F/X toolkit data.  It is
; called only once by languages that do not permit static data (like
; Turbo Pascal).
;
; Calling: fxInitData()
;
;

;Define variable locations on the stack, depending on model
idparm    equ       0

;Define local variables
idret     equ       <[bp-2]>                ;return code
idlocal   equ       2                       ;Total local space needed

fxInitData          PROC FAR

          @Entry    idlocal                 ;Set up frame and save regs

          @SetRet   idret,fxSUCCESS         ;Assume successful

          mov       fxEffect,fxWIPE         ;Effect default
          mov       fxDelay,MINDELAY        ;Default delay
          mov       fxGrain,8               ;Default grain
          mov       fx100th,500             ;Current 1/100 loop delay

          mov       fxVinit,pcxFALSE        ;Jump table uninitialized

          mov       fxKeyChk,pcxFALSE       ;No Key checking

fxID_exit:

          @Exit     idret,idparm            ;Return

fxInitData          ENDP

  @EndCode

          END

